@@ -1,7 +1,8 @@
-#' Read and recode UN Demographic and Health Surveys (DHS) individual data
+#' Read and recode Demographic and Health Surveys (DHS) individual data
 #'
 #' @param files vector: a character vector containing the paths for one or more Individual Recode DHS data files (see details)
 #' @param extra.vars vector: a character vector containing the names of variables to be retained from the raw data
+#' @param survey boolean: returns an unweighted data.frame if \code{FALSE}, or a weighted \link[survey]{svydesign} object if \code{TRUE}
 #' @param progress boolean: display a progress bar
 #'
 #' @details
@@ -11,11 +12,12 @@
 #'    files are available in SPSS, SAS, and Stata formats from \href{https://www.dhsprogram.com/}{https://www.dhsprogram.com/},
 #'    however access requires a \href{https://dhsprogram.com/data/Access-Instructions.cfm}{free application}. The `dhs()` function
 #'    reads one or more of these files, extracts and recodes selected variables useful for studying childfree adults and other
-#'    family statuses, then returns a single data frame.
+#'    family statuses, then returns either an unweighted data frame, or a weighted \link[survey]{svydesign} object that can be analyzed using the
+#'    \code{survey} package.
 #'
-#' Although access to DHS data requires an application, the DHS program provides \href{https://dhsprogram.com/data/Download-Model-Datasets.cfm}{model datasets}
+#' Although access to DHS data requires an application, the DHS program provides a \href{https://dhsprogram.com/data/Download-Model-Datasets.cfm}{model dataset}
 #'    for practice. The example provided below uses the model data file "ZZIR62FL.SAV", which contains fictitious women's data,
-#'    but has the same structure as real DHS data files. The example can be run without prior application for data access.
+#'    but has the same structure as a real DHS data file. The example can be run without prior application for data access.
 #'
 #' **Known issues**
 #'   * The SPSS-formatted files containing data from Gabon Recode 4 (GAIR41FL.SAV, GAMR41FL.SAV) and Turkey Recode 4 (TRIR41FL.SAV, TRMR41FL.SAV)
@@ -26,14 +28,29 @@
 #'   * Variables containing women's responses in the individual recode files begin with `v`, while variables containing men's responses in the
 #'     men recode files begin with `mv`. When applying `dhs()` to both female and male data, these are automatically harmonized. However, if
 #'     extra variables are requested using the `extra.vars` option, be sure to specify both names (e.g. `extra.vars = c("v201", "mv201")`).
+#'   * If \code{survey = TRUE}, then \code{(m)v021} and \code{(m)v023} are used as the cluster and strata indicators, respectively. This is
+#'     appropriate for most surveys, however there are a few exceptions. Additional information about analyzing DHS data using weights is
+#'     available \href{https://dhsprogram.com/data/Guide-to-DHS-Statistics/Analyzing_DHS_Data.htm}{here} and in the documentation provided
+#'     with the downloaded data files.
 #'
-#' @return A data frame containing variables described in the codebook available using \code{vignette("codebooks")}
+#' @return A data frame or weighted \link[survey]{svydesign} object containing variables described in the codebook available using \code{vignette("codebooks")}
+#' If you are offline, or if the requested data are otherwise unavailable, NULL is returned.
 #'
 #' @export
 #'
 #' @examples
-#' \donttest{data <- dhs(files = c("ZZIR62FL.SAV"), extra.vars = c("v201"))}
-dhs <- function(files, extra.vars = NULL, progress = TRUE) {
+#' \donttest{
+#' unweighted <- dhs(files = c("ZZIR62FL.SAV"), extra.vars = c("v201"))  #Request unweighted data
+#' if (!is.null(unweighted)) {  #If data was available...
+#' round(table(unweighted$famstat)/nrow(unweighted),3)  #Fraction of respondents w/ each family status
+#' }
+#'
+#' weighted <- dhs(files = c("ZZIR62FL.SAV"), survey = TRUE)  #Request weighted (example) data
+#' if (!is.null(weighted)) {  #If dtaa was available...
+#' survey::svymean(~famstat, weighted, na.rm = TRUE)  #Estimated prevalence of each family status
+#' }
+#' }
+dhs <- function(files, extra.vars = NULL, survey = FALSE, progress = TRUE) {
 
   if (length(files) > 1 & "ZZIR62FL.SAV" %in% files) {stop("Model data (file ZZIR62FL.SAV) should not be combined with files containing real data.")}
 
@@ -45,15 +62,16 @@ dhs <- function(files, extra.vars = NULL, progress = TRUE) {
   #Loop over each supplied data file
   for (file in 1:length(files)) {
 
-    #Increment progress bar
-    if (progress) {utils::setTxtProgressBar(pb,file)}
-
     #Import raw data
     if (files[file]=="ZZIR62FL.SAV") {  #Model file from https://dhsprogram.com/data/Download-Model-Datasets.cfm
+      if (!RCurl::url.exists("https://osf.io/download/hk23e")) {message("You are offline or sample DHS data is not available now. Try again later"); data <- NULL; return(data)}
       temp <- tempfile()
       utils::download.file(url = "https://osf.io/download/hk23e", destfile = temp)
       dat <- rio::import(temp, format = "sav")
-    } else {dat <- rio::import(files[file])}
+    } else {
+      if (progress) {utils::setTxtProgressBar(pb,file)}
+      dat <- rio::import(files[file])
+      }
     colnames(dat) <- tolower(colnames(dat))  #Make all variables lowercase
 
     #Check type of file
@@ -262,8 +280,10 @@ dhs <- function(files, extra.vars = NULL, progress = TRUE) {
                        "Trinidad and Tobago", "Tunisia", "Turkey", "Turkministan", "Uganda", "Ukraine", "Uzbekistan", "Vietnam", "Yemen", "Zambia", "Zimbabwe", "Papua New Guinea", "Fake Country")
     if (female) {dat$country <- country.names[match(substr(dat$v000,1,2), country.codes)]} else {dat$country <- country.names[match(substr(dat$mv000,1,2), country.codes)]}
 
-    #Sampling weight
+    #Weighting variables
     if (female) {dat$weight <- dat$v005/1000000} else {dat$weight <- dat$mv005/1000000}
+    if (female) {dat$cluster <- dat$v021} else {dat$cluster <- dat$mv021}
+    if (female) {dat$strata <- dat$v023} else {dat$strata <- dat$mv023}
 
     #Wave (called "Recode" in the DHS)
     if (female) {
@@ -309,13 +329,13 @@ dhs <- function(files, extra.vars = NULL, progress = TRUE) {
       dat <- dat[,c("cf_want", "cf_ideal", "famstat",  #Family status
                     "sex", "age", "education", "partnered", "residence", "employed",  #Demographics
                     "religion",  #Attitude
-                    "id", "country", "weight", "file", "survey", "wave", "year", "month",  #Design
+                    "id", "country", "weight", "cluster", "strata", "file", "survey", "wave", "year", "month",  #Design
                     extra.vars)]
     } else {
       dat <- dat[,c("cf_want", "cf_ideal", "famstat",  #Family status
                     "sex", "age", "education", "partnered", "residence", "employed",  #Demographics
                     "religion",  #Attitude
-                    "id", "country", "weight", "file", "survey", "wave", "year", "month")]  #Design
+                    "id", "country", "weight", "cluster", "strata", "file", "survey", "wave", "year", "month")]  #Design
     }
 
     #Start data file, or append to existing data file
@@ -325,6 +345,16 @@ dhs <- function(files, extra.vars = NULL, progress = TRUE) {
 
   #Finalize
   if (progress) {close(pb)}  #Close progress bar
-  class(data) <- c("data.frame", "childfree")
-  return(data)  #Export data
+
+  if (!survey) {
+    class(data) <- c("data.frame", "childfree")
+    return(data)
+  }
+
+  if (survey) {
+    data <- survey::svydesign(data = data, ids = ~cluster, strata = ~strata, weights = ~weight, nest = TRUE)
+    class(data) <- c("survey.design2", "survey.design", "childfree")
+    return(data)
+  }
+
 }
